@@ -7,7 +7,7 @@ a Python environment.
 :license: Apache License, Version 2.0, see LICENSE for details.
 """
 from __future__ import print_function
-
+import itertools
 import copy
 import numbers
 import os
@@ -37,6 +37,14 @@ class Namelist(OrderedDict):
     methods and properties are provided for working with Fortran namelists.
     """
 
+    class RepeatValue(object):
+        """Container class for output using repeat counters."""
+
+        def __init__(self, n, value):
+            """Create the RepeatValue object."""
+            self.repeats = n
+            self.value = value
+
     def __init__(self, *args, **kwds):
         """Create the Namelist object."""
         s_args = list(args)
@@ -75,6 +83,7 @@ class Namelist(OrderedDict):
         self._float_format = ''
         self._logical_repr = {False: '.false.', True: '.true.'}
         self._index_spacing = False
+        self._repeat_counter = False
 
         # Namelist group spacing flag
         self._newline = False
@@ -151,8 +160,11 @@ class Namelist(OrderedDict):
     def column_width(self):
         """Set the maximum number of characters per line of the namelist file.
 
+        :type: ``int``
+        :default: 72
+
         Tokens longer than ``column_width`` are allowed to extend past this
-        limit.  (Default: 72)
+        limit.
         """
         return self._column_width
 
@@ -167,13 +179,105 @@ class Namelist(OrderedDict):
         else:
             raise TypeError('Column width must be a nonnegative integer.')
 
+    @property
+    def default_start_index(self):
+        """Set the default start index for vectors with no explicit index.
+
+        :type: ``int``, ``None``
+        :default: ``None``
+
+        When the `default_start_index` is set, all vectors without an explicit
+        start index are assumed to begin with `default_start_index`.  This
+        index is shown when printing the namelist output.
+
+        If set to `None`, then no start index is assumed and is left as
+        implicit for any vectors undefined in `start_index`.
+        """
+        return self._default_start_index
+
+    @default_start_index.setter
+    def default_start_index(self, value):
+        if not isinstance(value, int):
+            raise TypeError('default_start_index must be an integer.')
+        self._default_start_index = value
+
+    @property
+    def end_comma(self):
+        """Append commas to the end of namelist variable entries.
+
+        :type: ``bool``
+        :default: ``False``
+
+        Fortran will generally disregard any commas separating variable
+        assignments, and the default behaviour is to omit these commas from the
+        output.  Enabling this flag will append commas at the end of the line
+        for each variable assignment.
+        """
+        return self._end_comma
+
+    @end_comma.setter
+    def end_comma(self, value):
+        """Validate and set the comma termination flag."""
+        if not isinstance(value, bool):
+            raise TypeError('end_comma attribute must be a logical type.')
+        self._end_comma = value
+
+    @property
+    def false_repr(self):
+        """Set the string representation of logical false values.
+
+        :type: ``str``
+        :default: ``'.false.'``
+
+        This is equivalent to the first element of ``logical_repr``.
+        """
+        return self._logical_repr[0]
+
+    @false_repr.setter
+    def false_repr(self, value):
+        """Validate and set the logical false representation."""
+        if isinstance(value, str):
+            if not (value.lower().startswith('f') or
+                    value.lower().startswith('.f')):
+                raise ValueError("Logical false representation must start "
+                                 "with 'F' or '.F'.")
+            else:
+                self._logical_repr[0] = value
+        else:
+            raise TypeError('Logical false representation must be a string.')
+
+    @property
+    def float_format(self):
+        """Set the namelist floating point format.
+
+        :type: ``str``
+        :default: ``''``
+
+        The property sets the format string for floating point numbers,
+        following the format expected by the Python ``format()`` function.
+        """
+        return self._float_format
+
+    @float_format.setter
+    def float_format(self, value):
+        """Validate and set the upper case flag."""
+        if isinstance(value, str):
+            # Duck-test the format string; raise ValueError on fail
+            '{0:{1}}'.format(1.23, value)
+
+            self._float_format = value
+        else:
+            raise TypeError('Floating point format code must be a string.')
+
     @property
     def indent(self):
         r"""Set the whitespace indentation of namelist entries.
 
+        :type: ``int``, ``str``
+        :default: ``'    '`` (four spaces)
+
         This can be set to an integer, denoting the number of spaces, or to an
         explicit whitespace character, such as a tab (``\t``).
-        (Default: 4)
         """
         return self._indent
 
@@ -199,27 +303,13 @@ class Namelist(OrderedDict):
             raise TypeError('Indentation must be specified by string or space '
                             'width.')
 
-    @property
-    def end_comma(self):
-        """Append commas to the end of namelist variable entries.
-
-        Fortran will generally disregard any commas separating variable
-        assignments, and the default behaviour is to omit these commas from the
-        output.  Enabling this flag will append commas at the end of the line
-        for each variable assignment.
-        """
-        return self._end_comma
-
-    @end_comma.setter
-    def end_comma(self, value):
-        """Validate and set the comma termination flag."""
-        if not isinstance(value, bool):
-            raise TypeError('end_comma attribute must be a logical type.')
-        self._end_comma = value
-
     @property
     def index_spacing(self):
-        """Apply a space between indexes of multidimensional vectors."""
+        """Apply a space between indexes of multidimensional vectors.
+
+        :type: ``bool``
+        :default: ``False``
+        """
         return self._index_spacing
 
     @index_spacing.setter
@@ -229,50 +319,20 @@ class Namelist(OrderedDict):
             raise TypeError('index_spacing attribute must be a logical type.')
         self._index_spacing = value
 
-    @property
-    def uppercase(self):
-        """Print group and variable names in uppercase."""
-        return self._uppercase
-
-    @uppercase.setter
-    def uppercase(self, value):
-        """Validate and set the uppercase flag."""
-        if not isinstance(value, bool):
-            raise TypeError('uppercase attribute must be a logical type.')
-        self._uppercase = value
-
-    @property
-    def float_format(self):
-        """Set the namelist floating point format.
-
-        The property sets the format string for floating point numbers,
-        following the format expected by the Python ``format()`` function.
-        """
-        return self._float_format
-
-    @float_format.setter
-    def float_format(self, value):
-        """Validate and set the upper case flag."""
-        if isinstance(value, str):
-            # Duck-test the format string; raise ValueError on fail
-            '{0:{1}}'.format(1.23, value)
-
-            self._float_format = value
-        else:
-            raise TypeError('Floating point format code must be a string.')
-
     # NOTE: This presumes that bools and ints are identical as dict keys
     @property
     def logical_repr(self):
         """Set the string representation of logical values.
 
+        :type: ``dict``
+        :default: ``{False: '.false.', True: '.true.'}``
+
         There are multiple valid representations of True and False values in
         Fortran.  This property sets the preferred representation in the
         namelist output.
 
         The properties ``true_repr`` and ``false_repr`` are also provided as
-        interfaces to the ``logical_repr`` tuple.
-        (Default: ``.false., .true.``)
+        interfaces to the elements of ``logical_repr``.
         """
         return self._logical_repr
 
@@ -292,6 +352,9 @@ class Namelist(OrderedDict):
     def true_repr(self):
         """Set the string representation of logical true values.
 
+        :type: ``str``
+        :default: ``.true.``
+
         This is equivalent to the second element of ``logical_repr``.
         """
         return self._logical_repr[1]
@@ -309,31 +372,13 @@ class Namelist(OrderedDict):
         else:
             raise TypeError('Logical true representation must be a string.')
 
-    @property
-    def false_repr(self):
-        """Set the string representation of logical false values.
-
-        This is equivalent to the first element of ``logical_repr``.
-        """
-        return self._logical_repr[0]
-
-    @false_repr.setter
-    def false_repr(self, value):
-        """Validate and set the logical false representation."""
-        if isinstance(value, str):
-            if not (value.lower().startswith('f') or
-                    value.lower().startswith('.f')):
-                raise ValueError("Logical false representation must start "
-                                 "with 'F' or '.F'.")
-            else:
-                self._logical_repr[0] = value
-        else:
-            raise TypeError('Logical false representation must be a string.')
-
     @property
     def start_index(self):
         """Set the starting index for each vector in the namelist.
 
+        :type: ``dict``
+        :default: ``{}``
+
         ``start_index`` is stored as a dict which contains the starting index
         for each vector saved in the namelist.  For the namelist ``vec.nml``
         shown below,
@@ -371,24 +416,24 @@ class Namelist(OrderedDict):
         self._start_index = value
 
     @property
-    def default_start_index(self):
-        """Set the default start index for vectors with no explicit index.
+    def repeat_counter(self):
+        """Return whether the namelist uses repeat counters for arrays.
 
-        When the `default_start_index` is set, all vectors without an explicit
-        start index are assumed to begin with `default_start_index`.  This
-        index is shown when printing the namelist output.
-        (Default: None)
+        If True, then arrays with repeated values will use repeat tokens.  For
+        example, the array ``[1, 2, 2, 2]`` will be written as ``1, 3*2``.
 
-        If set to `None`, then no start index is assumed and is left as
-        implicit for any vectors undefined in `start_index`.
+        :type: ``bool``
+        :default: ``False``
         """
-        return self._default_start_index
+        return self._repeat_counter
 
-    @default_start_index.setter
-    def default_start_index(self, value):
-        if not isinstance(value, int):
-            raise TypeError('default_start_index must be an integer.')
-        self._default_start_index = value
+    @repeat_counter.setter
+    def repeat_counter(self, value):
+        """Set whether array output should be done in repeat form."""
+        if isinstance(value, bool):
+            self._repeat_counter = value
+        else:
+            raise TypeError(r"repeat must be of type ``bool``")
 
     def write(self, nml_path, force=False, sort=False):
         """Write Namelist to a Fortran 90 namelist file.
@@ -407,6 +452,22 @@ class Namelist(OrderedDict):
             if not nml_is_file:
                 nml_file.close()
 
+    @property
+    def uppercase(self):
+        """Print group and variable names in uppercase.
+
+        :type: ``bool``
+        :default: ``False``
+        """
+        return self._uppercase
+
+    @uppercase.setter
+    def uppercase(self, value):
+        """Validate and set the uppercase flag."""
+        if not isinstance(value, bool):
+            raise TypeError('uppercase attribute must be a logical type.')
+        self._uppercase = value
+
     def patch(self, nml_patch):
         """Update the namelist from another partial or full namelist.
 
@@ -467,8 +528,7 @@ class Namelist(OrderedDict):
             v_start = grp_vars.start_index.get(v_name, None)
 
             for v_str in self._var_strings(v_name, v_val, v_start=v_start):
-                nml_line = self.indent + '{0}'.format(v_str)
-                print(nml_line, file=nml_file)
+                print(v_str, file=nml_file)
 
         print('/', file=nml_file)
 
@@ -580,46 +640,73 @@ class Namelist(OrderedDict):
                 v_idx_repr = ''
 
             # Split output across multiple lines (if necessary)
-
+            v_header = self.indent + v_name + v_idx_repr + ' = '
             val_strs = []
+            val_line = v_header
 
-            val_line = ''
-            for v_val in v_values:
-
-                v_header = v_name + v_idx_repr + ' = '
+            if self._repeat_counter:
+                v_values = list(
+                    self.RepeatValue(len(list(x)), val)
+                    for val, x in itertools.groupby(v_values)
+                )
+            for i_val, v_val in enumerate(v_values):
                 # Increase column width if the header exceeds this value
-                if len(self.indent + v_header) >= self.column_width:
-                    column_width = len(self.indent + v_header) + 1
+                if len(v_header) >= self.column_width:
+                    column_width = len(v_header) + 1
                 else:
                     column_width = self.column_width
 
-                v_width = column_width - len(self.indent + v_header)
+                if len(val_line) < column_width:
+                    # NOTE: We allow non-strings to extend past the column
+                    #   limit, but strings will be split as needed.
+                    v_str = self._f90repr(v_val)
+                    if isinstance(v_val, str):
+                        idx = column_width - len(val_line)
 
-                if len(val_line) < v_width:
-                    val_line += self._f90repr(v_val) + ', '
+                        # Split the line along idx until we either exceed the
+                        #   column width, or read the end of the string.
+                        while True:
+                            v_l, v_r = v_str[:idx], v_str[idx:]
+                            val_line += v_l
 
-                if len(val_line) >= v_width:
+                            if i_val < len(v_values) - 1 or self.end_comma:
+                                val_line += ', '
+
+                            if len(val_line) >= column_width:
+                                val_strs.append(val_line.rstrip())
+                                val_line = ''
+
+                            if v_r:
+                                v_str = v_r
+
+                                # Subsequent segments start on column zero
+                                idx = column_width
+                            else:
+                                break
+                    else:
+                        val_line += v_str
+                        if i_val < len(v_values) - 1 or self.end_comma:
+                            val_line += ', '
+
+                # Line break
+                if len(val_line) >= column_width:
+                    # Append current line to list of lines
                     val_strs.append(val_line.rstrip())
-                    val_line = ''
+                    # Start new line with space corresponding to header
+                    val_line = ' ' * len(v_header)
 
             # Append any remaining values
-            if val_line:
+            if val_line and not val_line.isspace():
                 val_strs.append(val_line.rstrip())
 
-            if val_strs:
-                if self.end_comma or v_values[-1] is None:
-                    pass
-                else:
-                    val_strs[-1] = val_strs[-1][:-1]
+            # Final null values must always precede a comma
+            if val_strs and v_values[-1] is None:
+                # NOTE: val_strs has been rstrip-ed so lead with a space
+                val_strs[-1] += ' ,'
 
             # Complete the set of values
             if val_strs:
-                var_strs.append('{0}{1} = {2}'
-                                ''.format(v_name, v_idx_repr,
-                                          val_strs[0]).strip())
-
-                for v_str in val_strs[1:]:
-                    var_strs.append(' ' * len(v_header) + v_str)
+                var_strs.extend(val_strs)
 
         return var_strs
 
@@ -678,7 +765,9 @@ class Namelist(OrderedDict):
 
     def _f90repr(self, value):
         """Convert primitive Python types to equivalent Fortran strings."""
-        if isinstance(value, bool):
+        if isinstance(value, self.RepeatValue):
+            return self._f90repeat(value)
+        elif isinstance(value, bool):
             return self._f90bool(value)
         elif isinstance(value, numbers.Integral):
             return self._f90int(value)
@@ -694,6 +783,14 @@ class Namelist(OrderedDict):
             raise ValueError('Type {0} of {1} cannot be converted to a Fortran'
                              ' type.'.format(type(value), value))
 
+    def _f90repeat(self, value):
+        """Return a Fortran 90 representation of a repeated value."""
+        if value.repeats == 1:
+            return self._f90repr(value.value)
+        else:
+            return "{0}*{1}".format(value.repeats,
+                                    self._f90repr(value.value))
+
     def _f90bool(self, value):
         """Return a Fortran 90 representation of a logical value."""
         return self.logical_repr[value]
