@@ -272,7 +272,14 @@ class Test(unittest.TestCase):
                 'a': {
                     'b': [{'c': 1, 'd': 2}]
                 }
-            }
+            },
+            'dtype_list_ooo_noidx': {
+                'a': [
+                    {'i': 123},
+                    {'i': 456},
+                    {'i': 789},
+                ],
+            },
         }
 
         self.dtype_case_nml = {
@@ -314,7 +321,7 @@ class Test(unittest.TestCase):
                         None, None, None, {'a': 1}
                     ]
                 }
-            }
+            },
         }
 
         self.bcast_nml = {
@@ -384,6 +391,12 @@ class Test(unittest.TestCase):
             }
         }
 
+        self.long_string_nml = {
+            'long_string_nml': {
+                's': ' '.join(12 * ['abcdefghijklmnopqrstuvwxyz'])
+            }
+        }
+
         self.ext_token_nml = {'ext_token_nml': {'x': 1}}
 
         self.list_patch_nml = {
@@ -396,6 +409,10 @@ class Test(unittest.TestCase):
             'repatch_nml': {
                 'x': [5, 6],
                 'y': {'z': 7}
+            },
+            'newgrp_nml': {
+                'a': 1,
+                'b': 2,
             }
         }
 
@@ -654,7 +671,12 @@ class Test(unittest.TestCase):
     def test_long_varname(self):
         test_nml = f90nml.read('types.nml')
         test_nml.column_width = 10
-        self.assert_write(test_nml, 'types.nml')
+        self.assert_write(test_nml, 'types_col10.nml')
+
+    def test_long_string(self):
+        test_nml = f90nml.read('long_string.nml')
+        self.assertEqual(self.long_string_nml, test_nml)
+        self.assert_write(test_nml, 'long_string_target.nml')
 
     def test_ext_token(self):
         test_nml = f90nml.read('ext_token.nml')
@@ -880,6 +902,15 @@ class Test(unittest.TestCase):
 
         self.assertRaises(TypeError, setattr, test_nml, 'uppercase', 'xyz')
 
+    def test_index_spacing(self):
+        test_nml = f90nml.read('multidim.nml')
+        test_nml.index_spacing = True
+
+        self.assertEqual(test_nml.index_spacing, True)
+        self.assert_write(test_nml, 'multidim_idx_space.nml')
+
+        self.assertRaises(TypeError, setattr, test_nml, 'index_spacing', 123)
+
     def test_float_format(self):
         test_nml = f90nml.read('float.nml')
         test_nml.float_format = '.3f'
@@ -1020,6 +1051,18 @@ class Test(unittest.TestCase):
         def test_numpy_write(self):
             self.assert_write(self.numpy_nml, 'numpy_types.nml')
 
+    def test_read_string(self):
+        test_nml = f90nml.reads(
+            '&types_nml'
+            ' v_integer = 1'
+            ' v_float = 1.0'
+            ' v_complex = (1.0, 2.0)'
+            ' v_logical = .true.'
+            ' v_string = \'Hello\''
+            '/'
+        )
+        self.assertEqual(self.types_nml, test_nml)
+
     # CLI tests
     def test_cli_help(self):
         cmd = ['f90nml']
@@ -1163,13 +1206,178 @@ class Test(unittest.TestCase):
         cmd = ['f90nml', 'types.yaml']
         source_str = self.get_cli_output(cmd, get_stderr=True)
 
-        target_str = ('f90nml: error: YAML module could not be found.\n'
+        target_str = (
+            'f90nml: error: YAML module could not be found.\n'
             '  To enable YAML support, install PyYAML or use the f90nml[yaml] '
-            'package.\n')
+            'package.\n'
+        )
         self.assertEqual(source_str, target_str)
 
         f90nml.cli.has_yaml = orig_has_yaml
 
+    def test_check_repeat_flag(self):
+        nml = f90nml.Namelist()
+        self.assertFalse(nml.repeat_counter)
+
+    def test_set_repeat_flag(self):
+        nml = f90nml.Namelist()
+        nml.repeat_counter = True
+        self.assertTrue(nml.repeat_counter)
+
+    def test_set_repeat_flag_incorrect(self):
+        nml = f90nml.Namelist()
+        with self.assertRaises(TypeError):
+            nml.repeat_counter = 'Hello'
+
+    # TODO fails in _var_strings
+    # def test_repeat_empty(self):
+        # nml_dict = {'a':{'b' : []}}
+        # nml = f90nml.Namelist(nml_dict)
+        # nml.repeat = True
+        # out = StringIO()
+        # print(nml, file=out)
+        # out.seek(0)
+        # line1 = out.readline()
+        # line2 = out.readline()
+        # self.assertEqual(line2.lstrip(), 'b = 1, 2, 3\n')
+
+    def test_repeat_scalar(self):
+        nml = f90nml.Namelist({'a': {'b': 1}})
+        nml.repeat_counter = True
+        out = StringIO()
+        print(nml, file=out)
+        out.seek(0)
+        line1 = out.readline()
+        line2 = out.readline()
+        self.assertEqual(line2.lstrip(), 'b = 1\n')
+
+    def test_repeat_single(self):
+        nml_dict = {'a': {'b': [1]}}
+        nml = f90nml.Namelist(nml_dict)
+        nml.repeat_counter = True
+        out = StringIO()
+        print(nml, file=out)
+        out.seek(0)
+        line1 = out.readline()
+        line2 = out.readline()
+        self.assertEqual(line2.lstrip(), 'b = 1\n')
+
+    def test_repeat_no_consecutive(self):
+        nml_dict = {'a': {'b': [1, 2, 3]}}
+        nml = f90nml.Namelist(nml_dict)
+        nml.repeat_counter = True
+        out = StringIO()
+        print(nml, file=out)
+        out.seek(0)
+        line1 = out.readline()
+        line2 = out.readline()
+        self.assertEqual(line2.lstrip(), 'b = 1, 2, 3\n')
+
+    def test_repeat_middle_consecutive(self):
+        nml_dict = {'a': {'b': [1, 2, 2, 3]}}
+        nml = f90nml.Namelist(nml_dict)
+        nml.repeat_counter = True
+        out = StringIO()
+        print(nml, file=out)
+        out.seek(0)
+        line1 = out.readline()
+        line2 = out.readline()
+        self.assertEqual(line2.lstrip(), 'b = 1, 2*2, 3\n')
+
+    def test_repeat_beginning_consecutive(self):
+        nml_dict = {'a': {'b': [1, 1, 2, 3]}}
+        nml = f90nml.Namelist(nml_dict)
+        nml.repeat_counter = True
+        out = StringIO()
+        print(nml, file=out)
+        out.seek(0)
+        line1 = out.readline()
+        line2 = out.readline()
+        self.assertEqual(line2.lstrip(), 'b = 2*1, 2, 3\n')
+
+    def test_repeat_end_consecutive(self):
+        nml_dict = {'a': {'b': [1, 2, 3, 3]}}
+        nml = f90nml.Namelist(nml_dict)
+        nml.repeat_counter = True
+        out = StringIO()
+        print(nml, file=out)
+        out.seek(0)
+        line1 = out.readline()
+        line2 = out.readline()
+        self.assertEqual(line2.lstrip(), 'b = 1, 2, 2*3\n')
+
+    def test_repeat_consecutive_repeating(self):
+        nml_dict = {'a': {'b': [1, 1, 2, 3, 1, 1, 1]}}
+        nml = f90nml.Namelist(nml_dict)
+        nml.repeat_counter = True
+        out = StringIO()
+        print(nml, file=out)
+        out.seek(0)
+        line1 = out.readline()
+        line2 = out.readline()
+        self.assertEqual(line2.lstrip(), 'b = 2*1, 2, 3, 3*1\n')
+
+    def test_repeat_all_repeating(self):
+        nml_dict = {'a': {'b': [1, 1, 1, 1, 1]}}
+        nml = f90nml.Namelist(nml_dict)
+        nml.repeat_counter = True
+        out = StringIO()
+        print(nml, file=out)
+        out.seek(0)
+        line1 = out.readline()
+        line2 = out.readline()
+        self.assertEqual(line2.lstrip(), 'b = 5*1\n')
+
+    def test_repeat_repeating_float(self):
+        nml_dict = {'a': {'b': [1.0, 1.0, 2.0, 0.3]}}
+        nml = f90nml.Namelist(nml_dict)
+        nml.repeat_counter = True
+        out = StringIO()
+        print(nml, file=out)
+        out.seek(0)
+        line1 = out.readline()
+        line2 = out.readline()
+        self.assertEqual(line2.lstrip(), 'b = 2*1.0, 2.0, 0.3\n')
+
+    def test_repeat_repeating_complex(self):
+        nml_dict = {'a': {'b': [1+2j, 1+2j, 3j]}}
+        nml = f90nml.Namelist(nml_dict)
+        nml.repeat_counter = True
+        out = StringIO()
+        print(nml, file=out)
+        out.seek(0)
+        line1 = out.readline()
+        line2 = out.readline()
+        self.assertEqual(line2.lstrip(), 'b = 2*(1.0, 2.0), (0.0, 3.0)\n')
+
+    def test_repeat_repeating_logical(self):
+        nml_dict = {'a': {'b': [True, True, False]}}
+        nml = f90nml.Namelist(nml_dict)
+        nml.repeat_counter = True
+        out = StringIO()
+        print(nml, file=out)
+        out.seek(0)
+        line1 = out.readline()
+        line2 = out.readline()
+        self.assertEqual(line2.lstrip(), "b = 2*.true., .false.\n")
+
+    # Failed namelist parsing
+    # NOTE: This is a very weak test, since '& x=1' / will pass
+    def test_grp_token_end(self):
+        self.assertRaises(ValueError, f90nml.reads, '&')
+
+    def test_no_group_name(self):
+        self.assertRaises(ValueError, f90nml.reads, '& /')
+
+    def test_empty_grp_no_end(self):
+        self.assertRaises(ValueError, f90nml.reads, '&end')
+
+    def test_string_grp_no_end(self):
+        self.assertRaises(ValueError, f90nml.reads, '&a_nml x = 1')
+
+    def test_file_grp_no_end(self):
+        self.assertRaises(ValueError, f90nml.read, 'grp_no_end.nml')
+
 
 if __name__ == '__main__':
     if os.path.isfile('tmp.nml'):
